/******************************************************************************

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#ifndef MATRIX_H
#define MATRIX_H

#include <iostream>
#include <vector>

/**
 * A templated matrix class.
 * The [] operator can be used as expected.
 */
template <class T>
class Matrix {
public:
     Matrix(int = 10, int = 10);
     Matrix(const Matrix&);
     ~Matrix();
     std::vector<T>& operator[](int);
     const std::vector<T>& operator[](int) const;
private:
     std::vector<T>* array_ptr;
     int max_elems;
};

template <class T>
Matrix<T>::Matrix(int n, int m) {
     //std::cout << "Constructing Matrix...\n";
     max_elems = n;     
     /*ISO-C++ does not permit initialization in this way:
       array_ptr = new std::vector<T>[n](m);
       , so we have to do it manually instead.
     */
     array_ptr = new std::vector<T>[n];
     for(int i = 0; i < n; i++)
         array_ptr[i].resize(m);
}

template <class T>
Matrix<T>::~Matrix() {
     delete [] array_ptr;
}

template <class T>
std::vector<T>& Matrix<T>::operator[](int index) {
     if (index >= 0 && index < max_elems) {
	  return array_ptr[index];
     }
     std::cerr << "Matrix: Row index out of bounds." << std::endl;;
     throw 0;
}

template <class T>
const std::vector<T>& Matrix<T>::operator[](int index) const {
     if (index >= 0 && index < max_elems) {
	  return array_ptr[index];
     }
     std::cerr << "Matrix: Row index out of bounds." << std::endl;;
     throw 0;
}

#endif
